@@ -1,17 +1,19 @@
 # TODO: Save to ini, toml
-import tomli
 from collections import Counter
 from configparser import ConfigParser
-from invisibleroads_macros_log import format_path
 from logging import getLogger
-from nbconvert import PythonExporter
-from nbformat import read as load_notebook, NO_CONVERT
 from os import environ, symlink
 from os.path import relpath, splitext
 from pathlib import Path
+from time import time
+
+import tomli
+from invisibleroads_macros_log import format_path
+from invisibleroads_macros_text import format_slug
+from nbconvert import PythonExporter
+from nbformat import read as load_notebook, NO_CONVERT
 from ruamel.yaml import YAML
 from ruamel.yaml.error import YAMLError
-from time import time
 
 from .. import __version__
 from ..constants import (
@@ -20,18 +22,21 @@ from ..constants import (
     AUTOMATION_VERSION,
     BATCH_ROUTE,
     MODE_NAMES,
+    PRINTER_BY_NAME,
     RUN_ROUTE,
-    STYLE_ROUTE)
+    STYLE_ROUTE,
+    VIEW_BY_NAME)
 from ..exceptions import (
     CrossComputeConfigurationError,
     CrossComputeConfigurationFormatError,
+    CrossComputeConfigurationNotImplementedError,
     CrossComputeError)
 from ..macros.package import is_equivalent_version
-from ..macros.web import format_slug
+from .printer import initialize_printer_by_name
 from .variable import (
     format_text,
     get_data_by_id_from_folder,
-    VARIABLE_VIEW_BY_NAME,
+    initialize_view_by_name,
     YIELD_DATA_BY_ID_BY_EXTENSION)
 
 
@@ -73,9 +78,10 @@ class AutomationDefinition(Definition):
             validate_batches,
             validate_datasets,
             validate_scripts,
-            validate_environment_variables,
+            validate_environment,
             validate_display_styles,
             validate_display_templates,
+            validate_display_pages,
             validate_print,
         ]
 
@@ -108,6 +114,13 @@ class AutomationDefinition(Definition):
         return get_template_text(
             template_definitions, automation_folder, variable_definitions)
 
+    def get_design_name(self, page_id):
+        design_name = DESIGN_NAMES_BY_PAGE_ID[page_id][0]
+        if page_id not in self.page_definition_by_id:
+            return design_name
+        page_definition = self.page_definition_by_id[page_id]
+        return page_definition.configuration.get('design', design_name)
+
     def update_datasets(self):
         automation_folder = self.folder
         for dataset_definition in self.dataset_definitions:
@@ -136,6 +149,7 @@ class VariableDefinition(Definition):
 class TemplateDefinition(Definition):
 
     def _initialize(self, kwargs):
+        self.automation_folder = kwargs['automation_folder']
         self.mode_name = kwargs.get('mode_name')
         self._validation_functions = [
             validate_template_identifiers,
@@ -159,7 +173,7 @@ class DatasetDefinition(Definition):
         self.automation_folder = kwargs['automation_folder']
         self._validation_functions = [
             validate_dataset_identifiers,
-            validate_dataset_configuration,
+            validate_dataset_reference,
         ]
 
 
@@ -184,7 +198,7 @@ class ScriptDefinition(Definition):
             old_path = folder / script_path
             script_path = '.' + str(script_path.with_suffix('.ipynb.py'))
             new_path = folder / script_path
-            L.info('exporting %s to %s', self.path, script_path)
+            L.info('exporting %s to %s in %s', self.path, script_path, folder)
             try:
                 script_text = PythonExporter().from_notebook_node(
                     load_notebook(old_path, NO_CONVERT))[0]
@@ -204,10 +218,20 @@ class StyleDefinition(Definition):
         ]
 
 
+class PageDefinition(Definition):
+
+    def _initialize(self, kwargs):
+        self._validation_functions = [
+            validate_page_identifiers,
+            validate_page_configuration,
+        ]
+
+
 class PrintDefinition(Definition):
 
     def _initialize(self, kwargs):
         self._validation_functions = [
+            validate_print_identifiers,
             validate_print_configuration,
             validate_header_footer_options,
             validate_page_number_options,
@@ -340,10 +364,10 @@ def validate_imports(configuration):
         _ for _ in automation_configurations if 'output' in _]
     assert_unique_values(
         [_.name for _ in automation_definitions],
-        'duplicate automation name {{x}}')
+        'duplicate automation name {x}')
     assert_unique_values(
         [_.slug for _ in automation_definitions],
-        'duplicate automation slug {{x}}')
+        'duplicate automation slug {x}')
     return {
         'automation_definitions': automation_definitions,
     }
@@ -370,9 +394,10 @@ def validate_variables(configuration):
 
 
 def validate_variable_views(configuration):
+    initialize_view_by_name()
     for view_name in configuration.___view_names:
         try:
-            View = VARIABLE_VIEW_BY_NAME[view_name]
+            View = VIEW_BY_NAME[view_name]
         except KeyError:
             raise CrossComputeConfigurationError(f'{view_name} not installed')
         environment_variable_ids = get_environment_variable_ids(
@@ -389,13 +414,8 @@ def validate_templates(configuration):
     for mode_name in MODE_NAMES:
         mode_configuration = get_dictionary(configuration, mode_name)
         template_definitions = [TemplateDefinition(
-            _, mode_name=mode_name,
+            _, automation_folder=automation_folder, mode_name=mode_name,
         ) for _ in get_dictionaries(mode_configuration, 'templates')]
-        for template_definition in template_definitions:
-            template_path = template_definition.path
-            if not (automation_folder / template_path).exists():
-                raise CrossComputeConfigurationError(
-                    f'could not find template {template_path}')
         assert_unique_values(
             [_.id for _ in template_definitions],
             f'duplicate template id {{x}} in {mode_name}')
@@ -415,20 +435,20 @@ def validate_batches(configuration):
             raw_batch_definition, automation_folder, variable_definitions))
     assert_unique_values(
         [_.folder for _ in batch_definitions],
-        'duplicate batch folder {{x}}')
+        'duplicate batch folder {x}')
     assert_unique_values(
         [_.name for _ in batch_definitions],
-        'duplicate batch name {{x}}')
+        'duplicate batch name {x}')
     assert_unique_values(
         [_.uri for _ in batch_definitions],
-        'duplicate batch uri {{x}}')
+        'duplicate batch uri {x}')
     return {
         'batch_definitions': batch_definitions,
         'run_definitions': [],
     }
 
 
-def validate_environment_variables(configuration):
+def validate_environment(configuration):
     environment_configuration = get_dictionary(configuration, 'environment')
     environment_variable_definitions = get_dictionaries(
         environment_configuration, 'variables')
@@ -436,8 +456,14 @@ def validate_environment_variables(configuration):
         environment_variable_definitions)
     if environment_variable_ids:
         L.debug('environment_variable_ids = %s', environment_variable_ids)
+    batch_concurrency_name = environment_configuration.get(
+        'batch', 'process').lower()
+    if batch_concurrency_name not in ('process', 'thread', 'single'):
+        raise CrossComputeConfigurationError(
+            f'"{batch_concurrency_name}" batch concurrency is not supported')
     return {
         'environment_variable_ids': environment_variable_ids,
+        'batch_concurrency_name': batch_concurrency_name,
     }
 
 
@@ -494,11 +520,13 @@ def validate_display_styles(configuration):
 
 
 def validate_display_templates(configuration):
-    display_dictionary = get_dictionary(configuration, 'display')
     template_path_by_id = {}
+    display_dictionary = get_dictionary(configuration, 'display')
+    automation_folder = configuration.folder
     for raw_template_definition in get_dictionaries(
             display_dictionary, 'templates'):
-        template_definition = TemplateDefinition(raw_template_definition)
+        template_definition = TemplateDefinition(
+            raw_template_definition, automation_folder=automation_folder)
         template_id = template_definition.id
         template_path_by_id[template_id] = template_definition.path
     return {
@@ -506,31 +534,46 @@ def validate_display_templates(configuration):
     }
 
 
-def validate_print(configuration):
-    print_dictionary = get_dictionary(configuration, 'print')
+def validate_display_pages(configuration):
+    display_dictionary = get_dictionary(configuration, 'display')
+    page_definitions = [PageDefinition(_) for _ in get_dictionaries(
+        display_dictionary, 'pages')]
+    page_definition_by_id = {_.id: _ for _ in page_definitions}
     return {
-        'print_definition': PrintDefinition(print_dictionary),
+        'page_definition_by_id': page_definition_by_id,
     }
 
 
-def validate_template_identifiers(template_definition):
+def validate_print(configuration):
+    print_definitions = [PrintDefinition(_) for _ in get_dictionaries(
+        configuration, 'prints')]
+    return {
+        'print_definitions': print_definitions,
+    }
+
+
+def validate_template_identifiers(template_dictionary):
     try:
-        template_path = template_definition['path']
+        template_path = template_dictionary['path']
     except KeyError as e:
         raise CrossComputeConfigurationError(f'{e} required for each template')
-    template_id = template_definition.get('id', template_path)
+    automation_folder = template_dictionary.automation_folder
+    template_path = Path(template_path)
+    if not (automation_folder / template_path).exists():
+        raise CrossComputeConfigurationError(
+            f'could not find template {template_path}')
     return {
-        'id': template_id,
-        'path': Path(template_path),
+        'id': template_dictionary.get('id', template_path.stem),
+        'path': template_path,
     }
 
 
-def validate_variable_identifiers(variable_definition):
+def validate_variable_identifiers(variable_dictionary):
     # TODO: Check that variable_id does not have quotes
     try:
-        variable_id = variable_definition['id']
-        view_name = variable_definition['view']
-        variable_path = variable_definition['path']
+        variable_id = variable_dictionary['id']
+        view_name = variable_dictionary['view']
+        variable_path = variable_dictionary['path']
     except KeyError as e:
         raise CrossComputeConfigurationError(f'{e} required for each variable')
     if relpath(variable_path).startswith('..'):
@@ -544,38 +587,40 @@ def validate_variable_identifiers(variable_definition):
     }
 
 
-def validate_variable_configuration(variable_definition):
+def validate_variable_configuration(variable_dictionary):
     variable_configuration = get_dictionary(
-        variable_definition, 'configuration')
+        variable_dictionary, 'configuration')
     return {
         'configuration': variable_configuration,
     }
 
 
-def validate_batch_identifiers(batch_definition):
-    is_run = batch_definition.is_run
+def validate_batch_identifiers(batch_dictionary):
+    is_run = batch_dictionary.is_run
     try:
-        folder = Path(get_scalar_text(batch_definition, 'folder'))
+        folder = Path(get_scalar_text(batch_dictionary, 'folder'))
     except KeyError as e:
         raise CrossComputeConfigurationError(f'{e} required for each batch')
-    name = get_scalar_text(batch_definition, 'name', folder.name)
-    slug = get_scalar_text(batch_definition, 'slug', name)
-    data_by_id = batch_definition.data_by_id
+    name = get_scalar_text(batch_dictionary, 'name', folder.name)
+    slug = get_scalar_text(batch_dictionary, 'slug', name)
+    data_by_id = batch_dictionary.data_by_id
     if data_by_id and not is_run:
         try:
-            folder = format_text(str(folder), data_by_id)
+            folder = format_text(folder, data_by_id)
             name = format_text(name, data_by_id)
             slug = format_text(slug, data_by_id)
+        except CrossComputeConfigurationNotImplementedError:
+            raise
         except CrossComputeConfigurationError as e:
-            batch_configuration = batch_definition.get('configuration', {})
+            batch_configuration = batch_dictionary.get('configuration', {})
             if 'path' in batch_configuration:
                 e.path = batch_configuration['path']
             raise
     d = {'folder': folder, 'name': name, 'slug': slug}
     if data_by_id:
         for k, v in d.items():
-            if k in batch_definition:
-                batch_definition[k] = v
+            if k in batch_dictionary:
+                batch_dictionary[k] = v
     if data_by_id is not None:
         if is_run:
             uri = RUN_ROUTE.format(run_slug=slug)
@@ -586,31 +631,31 @@ def validate_batch_identifiers(batch_definition):
     return d
 
 
-def validate_batch_configuration(batch_definition):
-    batch_reference = get_dictionary(batch_definition, 'reference')
-    batch_configuration = get_dictionary(batch_definition, 'configuration')
+def validate_batch_configuration(batch_dictionary):
+    batch_reference = get_dictionary(batch_dictionary, 'reference')
+    batch_configuration = get_dictionary(batch_dictionary, 'configuration')
     return {
         'reference': batch_reference,
         'configuration': batch_configuration,
     }
 
 
-def validate_dataset_identifiers(dataset_definition):
-    path = Path(dataset_definition.get('path', '').strip())
+def validate_dataset_identifiers(dataset_dictionary):
+    path = Path(dataset_dictionary.get('path', '').strip())
     return {
         'path': path,
     }
 
 
-def validate_dataset_configuration(dataset_definition):
-    automation_folder = dataset_definition.automation_folder
-    dataset_reference = get_dictionary(dataset_definition, 'reference')
+def validate_dataset_reference(dataset_dictionary):
+    automation_folder = dataset_dictionary.automation_folder
+    dataset_reference = get_dictionary(dataset_dictionary, 'reference')
     if 'path' in dataset_reference:
         source_path = Path(dataset_reference['path'].strip())
         if not (automation_folder / source_path).exists():
             raise CrossComputeConfigurationError(
                 f'could not find dataset reference path {source_path}')
-        target_path = dataset_definition.path
+        target_path = dataset_dictionary.path
         if target_path.exists() and not target_path.is_symlink():
             raise CrossComputeConfigurationError(
                 'refusing to overwrite existing dataset; please delete '
@@ -620,11 +665,11 @@ def validate_dataset_configuration(dataset_definition):
     }
 
 
-def validate_script_identifiers(script_definition):
-    folder = script_definition.get('folder', '.').strip()
+def validate_script_identifiers(script_dictionary):
+    folder = script_dictionary.get('folder', '.').strip()
 
-    if 'path' in script_definition:
-        path = Path(script_definition['path'].strip())
+    if 'path' in script_dictionary:
+        path = Path(script_dictionary['path'].strip())
         suffix = path.suffix
         if suffix not in ['.ipynb', '.py']:
             raise CrossComputeConfigurationError(
@@ -632,8 +677,8 @@ def validate_script_identifiers(script_definition):
     else:
         path = None
 
-    if 'command' in script_definition:
-        command = script_definition['command'].strip()
+    if 'command' in script_dictionary:
+        command = script_dictionary['command'].strip()
     else:
         command = None
 
@@ -644,9 +689,9 @@ def validate_script_identifiers(script_definition):
     }
 
 
-def validate_style_identifiers(style_definition):
-    uri = style_definition.get('uri', '').strip()
-    path = style_definition.get('path', '').strip()
+def validate_style_identifiers(style_dictionary):
+    uri = style_dictionary.get('uri', '').strip()
+    path = style_dictionary.get('path', '').strip()
     if not uri and not path:
         raise CrossComputeConfigurationError(
             'uri or path required for each style')
@@ -656,6 +701,55 @@ def validate_style_identifiers(style_definition):
     }
 
 
+def validate_page_identifiers(page_dictionary):
+    try:
+        page_id = page_dictionary['id']
+    except KeyError as e:
+        raise CrossComputeConfigurationError(f'{e} required for each page')
+    if page_id not in DESIGN_NAMES_BY_PAGE_ID:
+        raise CrossComputeConfigurationError(
+            f'{page_id} page not supported for page configuration')
+    return {
+        'id': page_id,
+    }
+
+
+def validate_page_configuration(page_dictionary):
+    page_configuration = page_dictionary.get('configuration', {})
+    page_id = page_dictionary['id']
+    design_name = page_configuration.get('design')
+    design_names = DESIGN_NAMES_BY_PAGE_ID[page_id]
+    if design_name not in design_names:
+        raise CrossComputeConfigurationError(
+            f'"{design_name}" design not supported for {page_id} page')
+    return {
+        'configuration': page_configuration,
+    }
+
+
+def validate_print_identifiers(print_dictionary):
+    initialize_printer_by_name()
+    print_format = print_dictionary.get('format', '').strip()
+    if print_format:
+        try:
+            PRINTER_BY_NAME[print_format]
+        except KeyError:
+            printer_names = PRINTER_BY_NAME.keys()
+            if printer_names:
+                extra_message = 'try ' + ' '.join(printer_names)
+            else:
+                extra_message = 'install crosscompute-printers-pdf'
+            raise CrossComputeConfigurationError(
+                f'{print_format} is not a supported printer; {extra_message}')
+    print_folder = Path(print_dictionary.get('folder', '')).expanduser()
+    print_name = print_dictionary.get('name', '')
+    return {
+        'format': print_format,
+        'folder': print_folder,
+        'name': print_name,
+    }
+
+
 def validate_print_configuration(print_dictionary):
     print_configuration = get_dictionary(
         print_dictionary, 'configuration')
@@ -816,4 +910,11 @@ def assert_unique_values(xs, message):
             raise CrossComputeConfigurationError(message.format(x=x))
 
 
+DESIGN_NAMES_BY_PAGE_ID = {
+    'automation': ['input', 'output', 'none'],
+    'input': ['flex-vertical', 'none'],
+    'output': ['flex-vertical', 'none'],
+    'log': ['flex-vertical', 'none'],
+    'debug': ['flex-vertical', 'none'],
+}
 L = getLogger(__name__)
