@@ -2,24 +2,28 @@ import csv
 import json
 import shutil
 from dataclasses import dataclass
-from importlib_metadata import entry_points
-from invisibleroads_macros_log import format_path
 from logging import getLogger
 from os.path import basename, exists
 from string import Template
 
+from importlib_metadata import entry_points
+from invisibleroads_macros_log import format_path
+from invisibleroads_macros_text import format_name, format_slug
+
 from ..constants import (
-    FUNCTION_BY_NAME,
     MAXIMUM_FILE_CACHE_LENGTH,
     TEMPLATES_FOLDER,
-    VARIABLE_ID_PATTERN)
+    VARIABLE_ID_PATTERN,
+    VIEW_BY_NAME)
 from ..exceptions import (
     CrossComputeConfigurationError,
+    CrossComputeConfigurationNotImplementedError,
     CrossComputeDataError)
 from ..macros.disk import FileCache
 from ..macros.package import import_attribute
 from ..macros.web import (
-    escape_quotes_html, escape_quotes_js)
+    escape_quotes_html,
+    escape_quotes_js)
 from .interface import Batch
 
 
@@ -29,6 +33,7 @@ class Element():
     id: str
     base_uri: str
     mode_name: str
+    design_name: str
     for_print: bool
     function_names: list[str]
 
@@ -48,7 +53,7 @@ class VariableView():
     def get_from(Class, variable_definition):
         view_name = variable_definition.view_name
         try:
-            View = VARIABLE_VIEW_BY_NAME[view_name]
+            View = VIEW_BY_NAME[view_name]
         except KeyError:
             L.error('%s view not installed', view_name)
             View = Class
@@ -62,13 +67,22 @@ class VariableView():
             render = self.render_input
         else:
             render = self.render_output
-        return render(b, x)
+        page_dictionary = render(b, x)
+        main_text = page_dictionary['main_text']
+        if x.design_name != 'none':
+            if main_text.endswith('</a>') or main_text.endswith('</span>'):
+                tag_name = 'span'
+            else:
+                tag_name = 'div'
+            page_dictionary['main_text'] = '<%s class="_view">%s</%s>' % (
+                tag_name, main_text, tag_name)
+        return page_dictionary
 
     def render_input(self, b: Batch, x: Element):
         return {
             'css_uris': [],
             'js_uris': [],
-            'body_text': '',
+            'main_text': '',
             'js_texts': [],
         }
 
@@ -76,7 +90,7 @@ class VariableView():
         return {
             'css_uris': [],
             'js_uris': [],
-            'body_text': '',
+            'main_text': '',
             'js_texts': [],
         }
 
@@ -91,25 +105,27 @@ class LinkView(VariableView):
         element_id = x.id
         data_uri = b.get_data_uri(variable_definition, x)
         c = b.get_variable_configuration(variable_definition)
-        name = c.get('name', basename(self.variable_path))
-        text = c.get('text', name)
-        body_text = (
+        file_name = c.get('file-name', basename(self.variable_path))
+        link_text = c.get('link-text', file_name)
+        main_text = (
             f'<a id="{element_id}" href="{data_uri}" '
-            f'class="{self.mode_name} {self.view_name} {variable_id}" '
-            f'download="{escape_quotes_html(name)}">'
-            f'{text}</a>')
+            f'class="_{self.mode_name} _{self.view_name} {variable_id}" '
+            f'download="{escape_quotes_html(file_name)}">'
+            f'{link_text}</a>')
+        if x.design_name not in ['none']:
+            main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
-            LINK_HEADER,
-            LINK_OUTPUT.substitute({
+            LINK_JS_HEADER,
+            LINK_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
-                'text': escape_quotes_js(text),
+                'link_text': escape_quotes_js(link_text),
             }),
         ]
         return {
             'css_uris': [],
             'js_uris': [],
-            'body_text': body_text,
+            'main_text': main_text,
             'js_texts': js_texts,
         }
 
@@ -118,7 +134,9 @@ class StringView(VariableView):
 
     view_name = 'string'
     input_type = 'text'
-    function_by_name = FUNCTION_BY_NAME
+    function_by_name = {
+        'title': str.title,
+    }
 
     def get_value(self, b: Batch):
         variable_definition = self.variable_definition
@@ -132,23 +150,29 @@ class StringView(VariableView):
         return value
 
     def render_input(self, b: Batch, x: Element):
-        view_name = self.view_name
+        variable_definition = self.variable_definition
         variable_id = self.variable_id
+        view_name = self.view_name
+        element_id = x.id
         value = self.get_value(b)
-        body_text = (
-            f'<input id="{x.id}" '
-            f'class="{self.mode_name} {view_name} {variable_id}" '
-            f'value="{escape_quotes_html(value)}" type="{self.input_type}" '
-            f'data-view="{view_name}" data-id="{variable_id}">')
+        c = b.get_variable_configuration(variable_definition)
+        main_text = STRING_HTML_INPUT.substitute({
+            'element_id': element_id,
+            'mode_name': self.mode_name,
+            'view_name': view_name,
+            'variable_id': variable_id,
+            'value': escape_quotes_html(value),
+            'input_type': self.input_type,
+        })
+        if x.design_name not in ['none']:
+            main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
-            STRING_INPUT.substitute({
-                'view_name': view_name,
-            }),
+            STRING_JS_INPUT.substitute({'view_name': view_name}),
         ]
         return {
             'css_uris': [],
             'js_uris': [],
-            'body_text': body_text,
+            'main_text': main_text,
             'js_texts': js_texts,
         }
 
@@ -163,13 +187,16 @@ class StringView(VariableView):
         variable_id = self.variable_id
         element_id = x.id
         data_uri = b.get_data_uri(variable_definition, x)
-        body_text = (
+        c = b.get_variable_configuration(variable_definition)
+        main_text = (
             f'<span id="{x.id}" '
-            f'class="{self.mode_name} {self.view_name} {self.variable_id}">'
+            f'class="_{self.mode_name} _{self.view_name} {self.variable_id}">'
             f'{value}</span>')
+        if x.design_name not in ['none']:
+            main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
-            STRING_HEADER,
-            STRING_OUTPUT.substitute({
+            STRING_JS_HEADER,
+            STRING_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
                 'data_uri': data_uri,
@@ -178,7 +205,7 @@ class StringView(VariableView):
         return {
             'css_uris': [],
             'js_uris': [],
-            'body_text': body_text,
+            'main_text': main_text,
             'js_texts': js_texts,
         }
 
@@ -216,30 +243,31 @@ class TextView(StringView):
 
     def render_input(self, b: Batch, x: Element):
         variable_definition = self.variable_definition
-        element_id = x.id
-        data_uri = b.get_data_uri(variable_definition, x)
-        view_name = self.view_name
         variable_id = self.variable_id
-        body_text = (
-            f'<textarea id="{x.id}" '
-            f'class="{self.mode_name} {view_name} {variable_id}" '
-            f'data-view="{view_name}" data-id="{variable_id}" disabled>'
-            '</textarea>')
+        view_name = self.view_name
+        element_id = x.id
+        c = b.get_variable_configuration(variable_definition)
+        main_text = TEXT_HTML_INPUT.substitute({
+            'element_id': element_id,
+            'mode_name': self.mode_name,
+            'view_name': view_name,
+            'variable_id': variable_id,
+        })
+        if x.design_name not in ['none']:
+            main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
-            STRING_HEADER,
-            STRING_INPUT.substitute({
-                'view_name': view_name,
-            }),
-            TEXT_HEADER,
-            TEXT_INPUT.substitute({
+            STRING_JS_HEADER,
+            STRING_JS_INPUT.substitute({'view_name': view_name}),
+            TEXT_JS_HEADER,
+            TEXT_JS_INPUT.substitute({
                 'element_id': element_id,
-                'data_uri': data_uri,
+                'data_uri': b.get_data_uri(variable_definition, x),
             }),
         ]
         return {
             'css_uris': [],
             'js_uris': [],
-            'body_text': body_text,
+            'main_text': main_text,
             'js_texts': js_texts,
         }
 
@@ -248,13 +276,16 @@ class TextView(StringView):
         variable_id = self.variable_id
         element_id = x.id
         data_uri = b.get_data_uri(variable_definition, x)
-        body_text = (
+        c = b.get_variable_configuration(variable_definition)
+        main_text = (
             f'<span id="{x.id}" '
-            f'class="{self.mode_name} {self.view_name} {self.variable_id}">'
+            f'class="_{self.mode_name} _{self.view_name} {self.variable_id}">'
             '</span>')
+        if x.design_name not in ['none']:
+            main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
-            STRING_HEADER,
-            TEXT_OUTPUT.substitute({
+            STRING_JS_HEADER,
+            TEXT_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
                 'data_uri': data_uri,
@@ -263,7 +294,7 @@ class TextView(StringView):
         return {
             'css_uris': [],
             'js_uris': [],
-            'body_text': body_text,
+            'main_text': main_text,
             'js_texts': js_texts,
         }
 
@@ -280,14 +311,17 @@ class MarkdownView(TextView):
         variable_id = self.variable_id
         element_id = x.id
         data_uri = b.get_data_uri(variable_definition, x)
-        body_text = (
+        c = b.get_variable_configuration(variable_definition)
+        main_text = (
             f'<span id="{x.id}" '
-            f'class="{self.mode_name} {self.view_name} {self.variable_id}">'
+            f'class="_{self.mode_name} _{self.view_name} {self.variable_id}">'
             '</span>')
+        if x.design_name not in ['none']:
+            main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
-            STRING_HEADER,
-            MARKDOWN_HEADER,
-            MARKDOWN_OUTPUT.substitute({
+            STRING_JS_HEADER,
+            MARKDOWN_JS_HEADER,
+            MARKDOWN_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
                 'data_uri': data_uri,
@@ -296,7 +330,7 @@ class MarkdownView(TextView):
         return {
             'css_uris': [],
             'js_uris': self.js_uris,
-            'body_text': body_text,
+            'main_text': main_text,
             'js_texts': js_texts,
         }
 
@@ -310,14 +344,17 @@ class ImageView(VariableView):
         variable_id = self.variable_id
         element_id = x.id
         data_uri = b.get_data_uri(variable_definition, x)
-        body_text = (
+        c = b.get_variable_configuration(variable_definition)
+        main_text = (
             f'<img id="{x.id}" '
-            f'class="{self.mode_name} {self.view_name} {variable_id}" '
+            f'class="_{self.mode_name} _{self.view_name} {variable_id}" '
             f'src="{data_uri}" alt="">')
         # TODO: Show spinner onerror
+        if x.design_name not in ['none']:
+            main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
-            IMAGE_HEADER,
-            IMAGE_OUTPUT.substitute({
+            IMAGE_JS_HEADER,
+            IMAGE_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
                 'data_uri': data_uri,
@@ -326,7 +363,7 @@ class ImageView(VariableView):
         return {
             'css_uris': [],
             'js_uris': [],
-            'body_text': body_text,
+            'main_text': main_text,
             'js_texts': js_texts,
         }
 
@@ -340,13 +377,16 @@ class TableView(VariableView):
         variable_id = self.variable_id
         element_id = x.id
         data_uri = b.get_data_uri(variable_definition, x)
-        body_text = (
+        c = b.get_variable_configuration(variable_definition)
+        main_text = (
             f'<table id="{element_id}" '
-            f'class="{self.mode_name} {self.view_name} {variable_id}">'
+            f'class="_{self.mode_name} _{self.view_name} {variable_id}">'
             '<thead/><tbody/></table>')
+        if x.design_name not in ['none']:
+            main_text = add_label_html(main_text, c, variable_id, element_id)
         js_texts = [
-            TABLE_HEADER,
-            TABLE_OUTPUT.substitute({
+            TABLE_JS_HEADER,
+            TABLE_JS_OUTPUT.substitute({
                 'variable_id': variable_id,
                 'element_id': element_id,
                 'data_uri': data_uri,
@@ -355,11 +395,17 @@ class TableView(VariableView):
         return {
             'css_uris': [],
             'js_uris': [],
-            'body_text': body_text,
+            'main_text': main_text,
             'js_texts': js_texts,
         }
 
 
+def initialize_view_by_name():
+    for entry_point in entry_points().select(group='crosscompute.views'):
+        VIEW_BY_NAME[entry_point.name] = import_attribute(entry_point.value)
+    return VIEW_BY_NAME
+
+
 def save_variable_data(target_path, data_by_id, variable_definitions):
     variable_data_by_id = get_variable_data_by_id(
         variable_definitions, data_by_id)
@@ -523,6 +569,7 @@ def get_variable_value_by_id(data_by_id):
 
 
 def format_text(text, data_by_id):
+    text = str(text)
     if not data_by_id:
         return text
 
@@ -535,14 +582,15 @@ def format_text(text, data_by_id):
         except KeyError:
             raise CrossComputeConfigurationError(
                 f'variable {variable_id} missing in batch configuration')
-        text = variable_data.get('value', '')
+        value = variable_data.get('value', '')
         try:
-            text = apply_functions(
-                text, expression_terms[1:], FUNCTION_BY_NAME)
+            value = apply_functions(value, expression_terms[1:], {
+                'slug': format_slug,
+                'title': str.title})
         except KeyError as e:
-            raise CrossComputeConfigurationError(
-                f'{e} function not supported in {text}')
-        return str(text)
+            raise CrossComputeConfigurationNotImplementedError(
+                f'{e} function not supported in "{text}"')
+        return str(value)
 
     return VARIABLE_ID_PATTERN.sub(f, text)
 
@@ -560,6 +608,22 @@ def apply_functions(value, function_names, function_by_name):
     return value
 
 
+def add_label_html(main_text, variable_configuration, variable_id, element_id):
+    label_text = get_label_text(variable_configuration, variable_id)
+    if label_text:
+        main_text = '<label for="%s">%s</label> %s' % (
+            element_id, label_text, main_text)
+    return main_text
+
+
+def get_label_text(variable_configuration, variable_id):
+    if 'label' in variable_configuration:
+        label_text = variable_configuration['label'] or ''
+    else:
+        label_text = format_name(variable_id)
+    return label_text.strip()
+
+
 def load_view_text(file_name):
     return open(TEMPLATES_FOLDER / file_name).read().strip()
 
@@ -567,34 +631,34 @@ def load_view_text(file_name):
 L = getLogger(__name__)
 
 
-LINK_HEADER = load_view_text('linkHeader.js')
-LINK_OUTPUT = Template(load_view_text('linkOutput.js'))
+LINK_JS_HEADER = load_view_text('linkHeader.js')
+LINK_JS_OUTPUT = Template(load_view_text('linkOutput.js'))
 
 
-STRING_INPUT = Template(load_view_text('stringInput.js'))
-STRING_HEADER = load_view_text('stringHeader.js')
-STRING_OUTPUT = Template(load_view_text('stringOutput.js'))
+STRING_HTML_INPUT = Template(load_view_text('stringInput.html'))
+STRING_JS_INPUT = Template(load_view_text('stringInput.js'))
+STRING_JS_HEADER = load_view_text('stringHeader.js')
+STRING_JS_OUTPUT = Template(load_view_text('stringOutput.js'))
 
 
-TEXT_HEADER = load_view_text('textHeader.js')
-TEXT_INPUT = Template(load_view_text('textInput.js'))
-TEXT_OUTPUT = Template(load_view_text('textOutput.js'))
+TEXT_HTML_INPUT = Template(load_view_text('textInput.html'))
+TEXT_JS_HEADER = load_view_text('textHeader.js')
+TEXT_JS_INPUT = Template(load_view_text('textInput.js'))
+TEXT_JS_OUTPUT = Template(load_view_text('textOutput.js'))
 
 
-MARKDOWN_HEADER = load_view_text('markdownHeader.js')
-MARKDOWN_OUTPUT = Template(load_view_text('markdownOutput.js'))
+MARKDOWN_JS_HEADER = load_view_text('markdownHeader.js')
+MARKDOWN_JS_OUTPUT = Template(load_view_text('markdownOutput.js'))
 
 
-IMAGE_HEADER = load_view_text('imageHeader.js')
-IMAGE_OUTPUT = Template(load_view_text('imageOutput.js'))
+IMAGE_JS_HEADER = load_view_text('imageHeader.js')
+IMAGE_JS_OUTPUT = Template(load_view_text('imageOutput.js'))
 
 
-TABLE_HEADER = load_view_text('tableHeader.js')
-TABLE_OUTPUT = Template(load_view_text('tableOutput.js'))
+TABLE_JS_HEADER = load_view_text('tableHeader.js')
+TABLE_JS_OUTPUT = Template(load_view_text('tableOutput.js'))
 
 
-VARIABLE_VIEW_BY_NAME = {_.name: import_attribute(
-    _.value) for _ in entry_points().select(group='crosscompute.views')}
 YIELD_DATA_BY_ID_BY_EXTENSION = {
     '.csv': yield_data_by_id_from_csv,
     '.txt': yield_data_by_id_from_txt,
