@@ -31,8 +31,7 @@ def do(arguments=None):
     except CrossComputeError as e:
         L.error(e)
         return
-    if is_port_in_use(args.port, with_log=True):
-        raise SystemExit
+    check_port(args.port)
     serve_with(automation, args)
 
 
@@ -117,6 +116,12 @@ def serve(
         pass
 
 
+def check_port(port):
+    if is_port_in_use(port):
+        L.error('port %s is in use; could not start server', port)
+        raise SystemExit
+
+
 L = getLogger(__name__)
 
 
