@@ -1,3 +1,3 @@
 registerElement('$variable_id', function() {
-  refreshLink('$element_id', '$text');
+  refreshLink('$element_id', '$link_text');
 });
