@@ -1,11 +1,13 @@
 import json
+
 from crosscompute.exceptions import CrossComputeDataError
-from crosscompute.macros.web import format_slug
 from crosscompute.routines.variable import (
     apply_functions,
     update_variable_data)
 from pytest import raises
 
+from invisibleroads_macros_text import format_slug
+
 
 def test_apply_functions():
     with raises(KeyError):
